﻿using System;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Attributes;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Commerce.Controllers
{
    [Authenticate, InRole("ADMIN")]
    public sealed class FixedTaxRuleController : ExtController
    {
        #region GET/POST Actions

        [HttpPost, PersistState]
        public ActionResult Create(string name,
            string[] commonFlags,
            string amount,
            string[] flags,
            string validFrom,
            string validTo)
        {
            try
            {
                var m = Svc.ProcessOperation("CreateFixedTaxRule", new
                {
                    Name = name,
                    CommonFlags = commonFlags.Merge(","),
                    Amount = Svc.Numeric(amount),
                    Flags = flags.Merge(","),
                    ValidFrom = Svc.DateTime(validFrom),
                    ValidTo = Svc.DateTime(validTo)
                });
                return RedirectToAction(Loc.Act("Edit").Ctrl("SalesRule").Set(new
                {
                    id = JDM.Int32(m.EntityID)
                }));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Create").Ctrl("SalesRule").Set(new
            {
                code = "FIXED_TAX"
            }));
        }

        [HttpPost, PersistState]
        public ActionResult Update(int id, 
            string name,
            string[] commonFlags,
            string amount,
            string[] flags,
            string validFrom,
            string validTo)
        {
            try
            {
                Svc.ProcessOperation("UpdateFixedTaxRule", new
                {
                    EntityID = id,
                    Name = name,
                    CommonFlags = commonFlags.Merge(","),
                    Amount = Svc.Numeric(amount),
                    Flags = flags.Merge(","),
                    ValidFrom = Svc.DateTime(validFrom),
                    ValidTo = Svc.DateTime(validTo)
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Ctrl("SalesRule").Set(new
            {
                id
            }));
        }

        #endregion
    }
}