﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Attributes;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Globalization.Controllers
{
    [Authenticate, InRole("ADMIN")]
    public sealed class CountryController : ExtController
    {
        #region GET/POST Actions

        [HttpGet, RestoreState]
        public ActionResult Index()
        {
            return View();
        }

        [HttpGet, RestoreState]
        public ActionResult Create()
        {
            return View();
        }

        [HttpPost, PersistState]
        public ActionResult Create(string code,
            string name,
            string dateFormat,
            string timeFormat,
            string numericFormat_DecimalSeparator,
            string numericFormat_GroupSeparator,
            string numericFormat_Format,
            string numericFormat_Pattern,
            string[] flags)
        {
            try
            {
                var m = Svc.ProcessOperation("CreateCountry", new
                {
                    Code = code,
                    Name = name,
                    DateFormat = dateFormat,
                    TimeFormat = timeFormat,
                    NumericFormat = new
                    {
                        DecimalSeparator = numericFormat_DecimalSeparator,
                        GroupSeparator = numericFormat_GroupSeparator,
                        Format = numericFormat_Format,
                        Pattern = numericFormat_Pattern
                    },
                    Flags = flags.Merge(",")
                });
                return RedirectToAction(Loc.Act("Edit").Set(new
                {
                    id = JDM.Int32(m.EntityID)
                }));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Create"));
        }

        [HttpGet, RestoreState]
        public ActionResult Edit(int id)
        {
            var m = Svc.ProcessOperation("GetCountry", new
            {
                EntityID = id
            }, new
            {
                DisclosureLevel = "Entire"
            });
            if(m == null)
                return HttpNotFound();
            return View(m);
        }

        [HttpPost, PersistState]
        public ActionResult Update(int id,
            string code,
            string name,
            string dateFormat,
            string timeFormat,
            string numericFormat_DecimalSeparator,
            string numericFormat_GroupSeparator,
            string numericFormat_Format,
            string numericFormat_Pattern,
            string[] flags)
        {
            try
            {
                Svc.ProcessOperation("UpdateCountry", new
                {
                    EntityID = id,
                    Code = code,
                    Name = name,
                    DateFormat = dateFormat,
                    TimeFormat = timeFormat,
                    NumericFormat = new
                    {
                        DecimalSeparator = numericFormat_DecimalSeparator,
                        GroupSeparator = numericFormat_GroupSeparator,
                        Format = numericFormat_Format,
                        Pattern = numericFormat_Pattern
                    },
                    Flags = flags.Merge(",")
                });
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }

            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        [HttpGet, PersistState]
        public ActionResult Delete(int id)
        {
            try
            {
                Svc.ProcessOperation("DeleteCountry", new
                {
                    EntityID = id
                });
                return RedirectToAction(Loc.Act("Index"));
            }
            catch(Exception ex)
            {
                ModelState.AddModelError("API", ex);
            }
            return RedirectToAction(Loc.Act("Edit").Set(new
            {
                id
            }));
        }

        #endregion

        #region AJAX Actions

        [HttpPost]
        public ActionResult Grid(int? page, int? rows, string sord, string sidx)
        {
            page = page.HasValue ? page.Value - 1 : 0;
            rows = rows.HasValue ? rows.Value : 30;
            sord = sord.Equals("desc") ? "Descending" : "Ascending";

            var m = Svc.ProcessOperation("SearchCountries", new 
            {
                Index = page * rows,
                Count = rows,
                SortBy = new[] 
                {
                    sidx
                },
                SortDir = sord
            }, new
            {
                DisclosureLevel = "Base"
            });
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                data.Add(new[] { 
                    JDM.Int32(i.EntityID),
                    JDM.String(i.Code),
                    JDM.String(i.Name)
                });
            }

            return Json(new
            {
                pageIndex = page + 1,
                pageCount = Math.Ceiling(JDM.Double(m.TotalCount) / rows),
                totalCount = JDM.Int64(m.TotalCount),
                data = data.ToArray()
            });
        }

        [HttpPost]
        public ActionResult List()
        {
            var m = Svc.ProcessOperation("GetCountryDictionary");
            var data = new List<Object>();

            foreach(var i in m.Items)
            {
                data.Add(new
                {
                    id = JDM.String(i.Key),
                    value = JDM.String(i.Value)
                });
            }
            return Json(data.ToArray());
        }

        #endregion
    }
}
