﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Web.Mvc;
using System.Xml.Linq;
using Xenta.Attributes;
using Xenta.Utils;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Blog.Controllers
{
    public sealed class PostController : ExtController
    {
        #region GET/POST Actions

        [HttpGet]
        public ActionResult Index(int? id,
            int? tagID,
            string attributeIDs,
            string searchTerm,
            string dtFrom,
            string dtTo,
            int? page)
        {
            dynamic m = new ExpandoObject();

            m.Author = id.HasValue 
                ? Svc.ProcessOperation("GetPublicationAuthor", new
                {
                    EntityID = id
                }, new
                {
                    DisclosureLevel = "Base"
                }) 
                : null;
            if(id.HasValue && m.Author == null)
                return HttpNotFound();

            page = page.HasValue ? page.Value : 0;
            var dtNow = DateTime.UtcNow;
            var publishedFrom = String.IsNullOrWhiteSpace(dtFrom)
                ? (DateTime?)null
                : Svc.Date(dtFrom);
            var publishedTo = String.IsNullOrWhiteSpace(dtTo)
                ? (DateTime?)null
                : Svc.Date(dtTo);
            bool authorMode = id.HasValue &&
                Svc.IsAuthenticated &&
                Svc.PublicationAuthor != null &&
                JDM.Int32(Svc.PublicationAuthor.EntityID) == id;
            if(!authorMode && (!publishedTo.HasValue || publishedTo > dtNow))
                publishedTo = dtNow;

            m.CountPerPage = Svc.Config(Par.In("WebPortal").Of(this).Group("Index").Name("Count"), 10);
            m.Posts = Svc.ProcessOperation("SearchPublications", new
            {
                Term = new
                {
                    Expression = String.IsNullOrWhiteSpace(searchTerm) ?
                        String.Empty :
                        String.Format("%{0}%", searchTerm)
                },
                LanguageID = authorMode ? JDM.Int32(Svc.Localization.Language.EntityID) : null,
                AuthorID = id,
                TagID = tagID,
                AttributeIDs = !String.IsNullOrWhiteSpace(attributeIDs) 
                    ? attributeIDs.Split(',').Select(x => x.ParseAs<Int32>()) 
                    : null,
                Index = page * m.CountPerPage,
                Count = m.CountPerPage,
                PublishedFrom = publishedFrom,
                PublishedTo = publishedTo,
                FlagMask = authorMode 
                    ? "BlogPost" 
                    : "BlogPost,Active",
                SortBy = new[] 
                {
                    "PublishedOn"
                },
                SortDir = "Descending"
            }, new
            {
                DisclosureLevel = "Extended",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            return View(m);
        }

        [HttpGet, RestoreState]
        public ActionResult Show(int id)
        {
            dynamic m = new ExpandoObject();

            m.Post = Svc.ProcessOperation("GetPublication", new
            {
                EntityID = id
            }, new
            {
                DisclosureLevel = "Entire",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            if(m.Post == null || !JDM.HasFlags(m.Post.Flags, "BlogPost"))
                return HttpNotFound();
            if(!JDM.HasFlags(m.Post.Flags, "Active") ||
                JDM.DateTime(m.Post.PublishedOn) > DateTime.UtcNow)
            {
                if(!Svc.IsAuthenticated ||
                    Svc.PublicationAuthor == null ||
                    JDM.Int32(Svc.PublicationAuthor.EntityID) != JDM.Int32(m.Post.Author.EntityID))
                {
                    return HttpNotFound();
                }
            }
            m.Author = Svc.ProcessOperation("GetPublicationAuthor", new
            {
                EntityID = JDM.Int32(m.Post.Author.EntityID)
            }, new
            {
                DisclosureLevel = "Base"
            });
            return View(m);
        }

        #endregion

        #region Child Actions

        [ChildActionOnly]
        public ActionResult _Archive(int? id, 
            int? year, 
            int? month, 
            int? day, 
            int? period)
        {
            string rptMimeType = String.Empty;
            long rptElapsedTime = 0;
            dynamic m = new ExpandoObject();
            m.Error = String.Empty;
            m.Data = null;
            m.AuthorID = id;

            try
            {
                using(var rptData = Svc.BuildReport("PublicationCountByMonthReport",
                    new Dictionary<String, String>
                    {
                        {"authorID", id.ToString()},
                        {"languageID", JDM.String(Svc.Localization.Language.EntityID)},
                        {"year", year.ToString()},
                        {"month", month.ToString()},
                        {"day", day.ToString()},
                        {"period", period.ToString()},
                        {"flagMask", "Active,BlogPost"}
                    },
                    out rptMimeType,
                    out rptElapsedTime))
                {
                    m.Data = XElement.Load(rptData);
                }
            }
            catch(Exception ex)
            {
                m.Error = ex.ToString();
            }
            m.ElapsedTime = rptElapsedTime;

            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Latest()
        {
            var m = Svc.ProcessOperation("SearchPublications", new
            {
                LanguageID = JDM.Int32(Svc.Localization.Language.EntityID),
                Count = Svc.Config(Par.In("WebPortal").Of(this).Group("_Latest").Name("Count"), 10),
                PublishedTo = DateTime.UtcNow,
                FlagMask = "Active,BlogPost",
                SortBy = new[] 
                {
                    "PublishedOn"
                },
                SortDir = "Descending"
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Comments(int id)
        {
            dynamic m = new ExpandoObject();

            m.EntityID = id;
            m.Comments = Svc.ProcessOperation("SearchPublicationComments", new
            {
                HolderID = id
            }, new
            {
                DisclosureLevel = "Extended"
            }).Items;
            return PartialView(m);
        }

        #endregion
    }
}
