﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Web.Mvc;
using Xenta.Utils;
using Xenta.Web.Controllers;

namespace Xenta.Web.Areas.Catalog.Controllers
{
    public sealed class ItemController : ExtController
    {
        #region GET/POST Actions

        [HttpGet]
        public ActionResult Index(int? page,
            string searchTerm,
            int? categoryID,
            int? supplierID,
            string attributeIDs,
            int? tagID,
            int? groupID,
            string sortBy,
            bool? sortDesc)
        {
            sortBy = StringHelper.EnsureNotNull(sortBy).Trim()
                .ToLowerInvariant();
            page = page.HasValue ? page.Value : 0;
            dynamic m = new ExpandoObject();

            m.CountPerPage = Svc.Config(Par.In("WebPortal").Of("Catalog").Of(this).Group("Index").Name("Count"), 12);
            m.Index = Svc.ProcessOperation("SearchCatalogItems", new
            {
                LanguageID = JDM.Int32(Svc.Localization.Language.EntityID),
                Term = new
                {
                    Expression = String.IsNullOrWhiteSpace(searchTerm) ?
                        String.Empty :
                        String.Format("%{0}%", searchTerm)
                },
                CategoryID = categoryID,
                SupplierID = supplierID,
                AttributeIDs = String.IsNullOrWhiteSpace(attributeIDs) ? null : attributeIDs.Split(',').Select(x => x.ParseAs<Int32>()),
                TagID = tagID,
                GroupID = groupID,
                Index = page * m.CountPerPage,
                Count = m.CountPerPage,
                FlagMask = "Active",
                SortBy = new[]
                {
                    !String.IsNullOrEmpty(sortBy) && sortBy.Equals("rating") 
                        ? "RatingAverage" 
                        : "CreatedOn"
                },
                SortDir = !sortDesc.HasValue || sortDesc.Value
                    ? "Descending"
                    : "Ascending"
            }, new
            {
                DisclosureLevel = "Extended",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            return View(m);
        }

        [HttpGet]
        public ActionResult Show(int id)
        {
            var m = Svc.ProcessOperation("GetCatalogItem", new
            {
                EntityID = id
            }, new
            {
                DisclosureLevel = "Entire",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            if(m == null)
                return HttpNotFound();
            return View(m);
        }

        #endregion

        #region Childs Actions

        [ChildActionOnly]
        public ActionResult _Filter(int? categoryID, 
            int? supplierID, 
            string attributeIDs, 
            int? tagID, 
            int? groupID)
        {
            dynamic m = new ExpandoObject();
            m.Selected = new ExpandoObject();
            m.Filter = new ExpandoObject();

            //category
            m.Selected.Category = !categoryID.HasValue ? null : Svc.ProcessOperation("GetCatalogCategory", new
            {
                EntityID = categoryID.Value
            }, new
            {
                DisclosureLevel = "Entire"
            });
            m.Filter.Categories = Svc.ProcessOperation("SearchCatalogCategories", new
            {
                ParentID = categoryID.HasValue ? categoryID.Value : 0,
                FlagMask = "Active"
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            }).Items;

            //supplier
            m.Selected.Supplier = !supplierID.HasValue ? null : Svc.ProcessOperation("GetCatalogSupplier", new
            {
                EntityID = supplierID.Value
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });

            //tag
            m.Selected.Tag = !tagID.HasValue ? null : Svc.ProcessOperation("GetCatalogTag", new
            {
                EntityID = tagID.Value
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });

            //group
            m.Selected.Group = !groupID.HasValue ? null : Svc.ProcessOperation("GetCatalogItemGroup", new
            {
                EntityID = groupID.Value
            }, new
            {
                DisclosureLevel = "Base",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });

            //attributes
            m.Selected.Attributes = new List<dynamic>();
            if(!String.IsNullOrWhiteSpace(attributeIDs))
            {
                foreach(var i in attributeIDs.Split(',').Select(x => x.ParseAs<Int32>()))
                {
                    m.Selected.Attributes.Add(Svc.ProcessOperation("GetLookupAttribute", new
                    {
                        EntityID = i
                    }, new
                    {
                        DisclosureLevel = "Base",
                        DisclosureOptions = new Dictionary<String, String>
                        {
                            { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                        }.Unload()
                    }));
                }
            }
            m.Filter.Attributes = m.Selected.Category != null 
                ? m.Selected.Category.Attributes 
                : Enumerable.Empty<dynamic>();
            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Promoted()
        {
            var m = Svc.ProcessOperation("SearchCatalogItems", new
            {
                LanguageID = JDM.Int32(Svc.Localization.Language.EntityID),
                Count = Svc.Config(Par.In("WebPortal").Of("Catalog").Of(this).Group("_Promoted").Name("Count"), 12),
                FlagMask = "Active,Promoted",
                SortBy = new[] 
                {
                    "CreatedOn"
                },
                SortDir = "Descending"
            }, new
            {
                DisclosureLevel = "Extended",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Latest(int? supplierID)
        {
            var m = Svc.ProcessOperation("SearchCatalogItems", new
            {
                LanguageID = JDM.Int32(Svc.Localization.Language.EntityID),
                SupplierID = supplierID,
                Count = Svc.Config(Par.In("WebPortal").Of("Catalog").Of(this).Group("_Latest").Name("Count"), 12),
                FlagMask = "Active",
                SortBy = new[] 
                {
                    "CreatedOn"
                },
                SortDir = "Descending"
            }, new
            {
                DisclosureLevel = "Extended",
                DisclosureOptions = new Dictionary<String, String>
                {
                    { "TranslatedTo", JDM.String(Svc.Localization.Language.Code) }
                }.Unload()
            });
            return PartialView(m);
        }

        [ChildActionOnly]
        public ActionResult _Comments(int id)
        {
            dynamic m = new ExpandoObject();

            m.EntityID = id;
            m.Comments = Svc.ProcessOperation("SearchCatalogItemComments", new
            {
                HolderID = id
            }, new
            {
                DisclosureLevel = "Extended"
            }).Items;
            return PartialView(m);
        }

        #endregion
    }
}
