﻿using System;
using System.Collections.Generic;
using NHibernate.Mapping.ByCode;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the group translation entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   + Base
    ///    | - Holder(of the Primary level)
    ///    | - Language(of the Keys level)
    ///    | - Name
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class TCatalogItemGroupEntity : TranslationEntity<CatalogItemGroupEntity>
    {
        #region Properties

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        public virtual string Name
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Name"] = Name;
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class TCatalogItemGroupMapping : EntityMapping<TCatalogItemGroupEntity>
    {
        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public TCatalogItemGroupMapping()
        {
            Table("fwk_TCatalogItemGroups");
            Id(x => x.EntityID, m =>
            {
                m.Column("TranslationID");
                m.Generator(Generators.Identity);
            });
            ManyToOne(x => x.Holder, m =>
            {
                m.Column("GroupID");
                m.ForeignKey("FK_fwk_TCatalogItemGroups_fwk_CatalogItemGroups");
                m.Index("IX_fwk_TCatalogItemGroups");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            ManyToOne(x => x.Language, m =>
            {
                m.Column("LanguageID");
                m.ForeignKey("FK_fwk_TCatalogItemGroups_fwk_Languages");
                m.Index("IX_fwk_TCatalogItemGroups");
                m.NotNullable(true);
                m.Fetch(FetchKind.Join);
            });
            Property(x => x.Name, m =>
            {
                m.NotNullable(true);
                m.Length(100);
                m.Column(c =>
                {
                    c.SqlType("nvarchar(100)");
                });
            });
            Property(x => x.CreatedOn, m =>
            {
                m.NotNullable(true);
            });
            Property(x => x.UpdatedOn, m =>
            {
                m.NotNullable(true);
            });
        }
    }
}
