﻿using System;
using System.Collections.Generic;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the customer address entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - EntityName
    ///   | - ContactName
    ///   | - PhoneNumber
    ///   | - FaxNumber
    ///   | - Email
    ///   | - Website
    ///   + Base
    ///    | - HolderID
    ///    | - Country(of the Base level)
    ///    | - Region(of the Base level)
    ///    | - City
    ///    | - Address1
    ///    | - Address2
    ///    | - PostalCode
    ///    | - ShortForm
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class CustomerAddressEntity : AddressEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual CustomerAddressFlags Flags
        {
            get
            {
                return (CustomerAddressFlags)FlagsValue;
            }
            set
            {
                FlagsValue = (int)value;
            }
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
            {
                obj["Flags"] = Flags.ToString();
            }
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class CustomerAddressMapping : SubEntityMapping<CustomerAddressEntity>
    {
        #region Constants

        /// <summary>
        /// The value to be used as the address entity discriminator.
        /// </summary>
        public const string DISCRIMINATOR_VALUE = "CUSTOMER";

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CustomerAddressMapping()
        {
            DiscriminatorValue(DISCRIMINATOR_VALUE);
        }

        #endregion
    }
}
