﻿using System;
using System.Collections.Generic;
using Xenta.Enums;

namespace Xenta.Entities
{
    /// <summary>
    /// Represents the seller comment entity.
    /// </summary>
    /// <remarks><pre>
    /// The entity regarding the disclosure level.
    /// 
    /// + Entire
    ///  + Extended
    ///   | - Text
    ///   + Base
    ///    | - HolderID
    ///    | - Account(of the Keys level)
    ///    | - Title
    ///    | - Flags
    ///    | - UpdatedOn
    ///    | - CreatedOn
    ///    + Keys
    ///     + Primary
    ///      | - EntityID
    /// 
    /// Note: upper level includes lower level.
    /// </pre></remarks>
    public class SellerCommentEntity : CommentEntity
    {
        #region Properties

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public virtual SellerCommentFlags Flags
        {
            get
            {
                return (SellerCommentFlags)FlagsValue;
            }
            set
            {
                FlagsValue = (int)value;
            }
        }

        #endregion

        #region Utilities

        protected override void Disclose(IDictionary<String, Object> obj,
            DisclosureLevel level,
            IDictionary<String, String> options)
        {
            base.Disclose(obj, level, options);
            if(level >= DisclosureLevel.Base)
                obj["Flags"] = Flags.ToString();
        }

        #endregion
    }

    /// <summary>
    /// The entity NHibernate mapping.
    /// </summary>
    public class SellerCommentMapping : SubEntityMapping<SellerCommentEntity>
    {
        #region Constants

        /// <summary>
        /// The value to be used as the generic attribute entity discriminator.
        /// </summary>
        public const string DISCRIMINATOR_VALUE = "SELLER";

        #endregion

        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public SellerCommentMapping()
        {
            DiscriminatorValue(DISCRIMINATOR_VALUE);
        }

        #endregion
    }
}
