﻿using System.Collections.Generic;
using Xenta.Api;
using Xenta.Entities;
using Xenta.Security;
using Xenta.Utils;

namespace Xenta.Operations
{
    #region __General__

    /// <summary>
    /// Creates an order attribute.
    /// </summary>
    public class CreateOrderAttribute : CreateGenericAttribute<OrderAttributeEntity>
    {
    }

    /// <summary>
    /// Creates an order attribute.
    /// </summary>
    public class UpdateOrderAttribute : UpdateGenericAttribute<OrderAttributeEntity>
    {
    }

    /// <summary>
    /// Sets an order attribute.
    /// </summary>
    public class SetOrderAttribute : SetGenericAttribute<OrderAttributeEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("SALES");
            }
        }

        #endregion
    }

    /// <summary>
    /// Deletes the order attribute.
    /// </summary>
    public class DeleteOrderAttribute : DeleteGenericAttribute<OrderAttributeEntity>
    {
    }

    /// <summary>
    /// Gets the order attribute by identifier.
    /// </summary>
    public class GetOrderAttribute : GetGenericAttribute<OrderAttributeEntity>
    {
    }

    /// <summary>
    /// Gets the order attribute by the key.
    /// </summary>
    public class GetOrderAttributeByKey : GetGenericAttributeByKey<OrderAttributeEntity>
    {
    }

    /// <summary>
    /// Gets the order attribute value by key.
    /// </summary>
    public class GetOrderAttributeValueByKey : GetGenericAttributeValueByKey<OrderAttributeEntity>,
        IApiOperation
    {
    }

    #endregion

    #region __Bulk__

    /// <summary>
    /// Updates the order attributes from dictionary.
    /// </summary>
    public class BulkUpdateOrderAttributes : BulkUpdateGenericAttributes<OrderAttributeEntity>,
        ISecuredOperation, IApiOperation
    {
        #region Properties

        /// <summary>
        /// Gets the collection of security policies.
        /// </summary>
        public IEnumerable<SecurityPolicy> SecurityPolicies
        {
            get
            {
                yield return new SecurityPolicy()
                    .IsAuthenticated()
                    .HasPermission("SALES");
            }
        }

        #endregion
    }

    #endregion
}
