﻿using Xenta.Entities;
using Xenta.Operations;

namespace Xenta.Utils
{
    /// <summary>
    /// Contains extension methods.
    /// </summary>
    public static partial class Extensions
    {
        #region IWorkingContext

        /// <summary>
        /// Gets the customer entity associated with the context.
        /// </summary>
        /// <param name="ctx">The context.</param>
        /// <param name="uow">The unit of work.</param>
        /// <returns>The entity or null.</returns>
        public static CustomerEntity Customer(this IWorkingContext ctx, IUnitOfWork uow)
        {
            var account = ctx.Account(uow);
            if(account == null)
                return null;
            return uow.Execute(new GetCustomerByAccount
            {
                AccountID = account.EntityID
            }).Result;
        }

        /// <summary>
        /// Gets the seller entity associated with the context.
        /// </summary>
        /// <param name="ctx">The context.</param>
        /// <param name="uow">The unit of work.</param>
        /// <returns>The entity or null.</returns>
        public static SellerEntity Seller(this IWorkingContext ctx, IUnitOfWork uow)
        {
            var account = ctx.Account(uow);
            if(account == null)
                return null;
            return uow.Execute(new GetSellerByAccount
            {
                AccountID = account.EntityID
            }).Result;
        }

        #endregion
    }
}
