﻿using System;
using Xenta.Entities;
using Xenta.Enums;
using Xenta.Utils;

namespace Xenta.Operations
{
    /// <summary>
    /// Creates a sales rule.
    /// </summary>
    public class CreateFixedDiscountRule : CreateSalesRule<FixedDiscountRuleEntity>
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public CreateFixedDiscountRule()
        {
            UsageNumber = -1;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or set the coupon code.
        /// </summary>
        public string CouponCode
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the usage number.
        /// </summary>
        public int UsageNumber
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the amount.
        /// </summary>
        public decimal Amount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public FixedDiscountRuleFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(Amount <= Decimal.Zero)
                return Err.Of<FixedDiscountRuleEntity>().NotValid("Amount");
            return base.Validate(uow);
        }

        protected override void Construct(IUnitOfWork uow, FixedDiscountRuleEntity entity)
        {
            base.Construct(uow, entity);

            entity.CouponCode = StringHelper.EnsureNotNull(CouponCode).TrimToLen(10);
            entity.UsageNumber = UsageNumber;
            entity.Amount = Amount;
            entity.Flags = Flags;
        }

        #endregion
    }

    /// <summary>
    /// Updates the sales rule.
    /// </summary>
    public class UpdateFixedDiscountRule : UpdateSalesRule<FixedDiscountRuleEntity>
    {
        #region Ctors

        /// <summary>
        /// Initializes a new class instance.
        /// </summary>
        public UpdateFixedDiscountRule()
        {
            UsageNumber = -1;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or set the coupon code.
        /// </summary>
        public string CouponCode
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the usage number.
        /// </summary>
        public int UsageNumber
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the amount.
        /// </summary>
        public decimal Amount
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flags.
        /// </summary>
        public FixedDiscountRuleFlags Flags
        {
            get;
            set;
        }

        #endregion

        #region Utilities

        protected override Err Validate(IUnitOfWork uow)
        {
            if(Amount <= Decimal.Zero)
                return Err.Of<FixedDiscountRuleEntity>().NotValid("Amount");
            return base.Validate(uow);
        }

        protected override void Revise(IUnitOfWork uow, FixedDiscountRuleEntity entity)
        {
            base.Revise(uow, entity);

            entity.CouponCode = StringHelper.EnsureNotNull(CouponCode).TrimToLen(10);
            entity.UsageNumber = UsageNumber;
            entity.Amount = Amount;
            entity.Flags = Flags;
        }

        #endregion
    }
}
