﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class CatalogCategoryTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name")]
        public void Can_CRUD(string name)
        {
            var t_category = Processor.Execute(new CreateCatalogCategoryTranslation
            {
                HolderID = Processor.Execute(new CreateCatalogCategory
                {
                    ParentID = 0
                }).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(t_category);
            t_category.Name.AreEqualIgnoringCase(name);
            t_category.AreEqual(Processor.Execute(new GetCatalogCategoryTranslation
            {
                EntityID = t_category.EntityID
            }).Result);

            name = name.Reverse();
            var upd_t_category = Processor.Execute(new UpdateCatalogCategoryTranslation
            {
                EntityID = t_category.EntityID,
                LanguageID = t_category.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_category);
            upd_t_category.AreEqual(t_category);
            upd_t_category.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteCatalogCategoryTranslation
            {
                EntityID = t_category.EntityID
            });
        }
    }
}
