﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class CatalogItemGroupTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name")]
        public void Can_CRUD(string name)
        {
            var t_group = Processor.Execute(new CreateCatalogItemGroupTranslation
            {
                HolderID = Processor.Execute(new CreateCatalogItemGroup
                {
                    SupplierID = Processor.Execute(new CreateCatalogSupplier()).Result.EntityID
                }).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(t_group);
            t_group.Name.AreEqualIgnoringCase(name);
            t_group.AreEqual(Processor.Execute(new GetCatalogItemGroupTranslation
            {
                EntityID = t_group.EntityID
            }).Result);

            name = name.Reverse();
            var upd_t_group = Processor.Execute(new UpdateCatalogItemGroupTranslation
            {
                EntityID = t_group.EntityID,
                LanguageID = t_group.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_group);
            upd_t_group.AreEqual(t_group);
            upd_t_group.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteCatalogItemGroupTranslation
            {
                EntityID = t_group.EntityID
            });
        }
    }
}
