﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class CatalogSupplierAddressTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("city", "address1", "address2", "entityName",
            "contactName", "postalCode", "phoneNumber",
            "faxNumber", "email", "website")]
        public void Can_CRUD(string city,
            string address1,
            string address2,
            string entityName,
            string contactName,
            string postalCode,
            string phoneNumber,
            string faxNumber,
            string email,
            string website)
        {
            #region Prepare

            var supplier = Processor.Execute(new CreateCatalogSupplier()).Result;
            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var region = Processor.Execute(new CreateRegion
            {
                Code = "xy",
                CountryID = country.EntityID
            }).Result;

            #endregion

            var address = Processor.Execute(new CreateCatalogSupplierAddress
            {
                HolderID = supplier.EntityID,
                CountryID = country.EntityID,
                RegionID = region.EntityID,
                City = city,
                Address1 = address1,
                Address2 = address2,
                EntityName = entityName,
                ContactName = contactName,
                PostalCode = postalCode,
                PhoneNumber = phoneNumber,
                FaxNumber = faxNumber,
                Email = email,
                Website = website
            }).Result;


            Assert.NotNull(address);
            address.City.AreEqualIgnoringCase(city);
            address.Address1.AreEqualIgnoringCase(address1);
            address.Address2.AreEqualIgnoringCase(address2);
            address.EntityName.AreEqualIgnoringCase(entityName);
            address.ContactName.AreEqualIgnoringCase(contactName);
            address.PostalCode.AreEqualIgnoringCase(postalCode);
            address.PhoneNumber.AreEqualIgnoringCase(phoneNumber);
            address.FaxNumber.AreEqualIgnoringCase(faxNumber);
            address.Email.AreEqualIgnoringCase(email);
            address.Website.AreEqualIgnoringCase(website);
            address.AreEqual(Processor.Execute(new GetCatalogSupplierAddress
            {
                EntityID = address.EntityID
            }).Result);

            city = city.Reverse();
            address1 = address1.Reverse();
            address2 = address2.Reverse();
            entityName = entityName.Reverse();
            contactName = contactName.Reverse();
            postalCode = postalCode.Reverse();
            phoneNumber = phoneNumber.Reverse();
            faxNumber = faxNumber.Reverse();
            email = email.Reverse();
            website = website.Reverse();
            var upd_address = Processor.Execute(new UpdateCatalogSupplierAddress
            {
                EntityID = address.EntityID,
                CountryID = country.EntityID,
                RegionID = region.EntityID,
                City = city,
                Address1 = address1,
                Address2 = address2,
                EntityName = entityName,
                ContactName = contactName,
                PostalCode = postalCode,
                PhoneNumber = phoneNumber,
                FaxNumber = faxNumber,
                Email = email,
                Website = website,
                Flags = CatalogSupplierAddressFlags.Active
            }).Result;


            Assert.NotNull(upd_address);
            upd_address.AreEqual(address);
            upd_address.City.AreEqualIgnoringCase(city);
            upd_address.Address1.AreEqualIgnoringCase(address1);
            upd_address.Address2.AreEqualIgnoringCase(address2);
            upd_address.EntityName.AreEqualIgnoringCase(entityName);
            upd_address.ContactName.AreEqualIgnoringCase(contactName);
            upd_address.PostalCode.AreEqualIgnoringCase(postalCode);
            upd_address.PhoneNumber.AreEqualIgnoringCase(phoneNumber);
            upd_address.FaxNumber.AreEqualIgnoringCase(faxNumber);
            upd_address.Email.AreEqualIgnoringCase(email);
            upd_address.Website.AreEqualIgnoringCase(website);
            upd_address.Flags.AreEqual(CatalogSupplierAddressFlags.Active);

            Processor.Execute(new DeleteCatalogSupplierAddress
            {
                EntityID = address.EntityID
            });
        }
    }
}
