﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class CatalogSupplierTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name", "shortDescription", "description")]
        public void Can_CRUD(string name, string shortDescription, string description)
        {
            var t_supplier = Processor.Execute(new CreateCatalogSupplierTranslation
            {
                HolderID = Processor.Execute(new CreateCatalogSupplier()).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Name = name,
                ShortDescription = shortDescription,
                Description = description
            }).Result;

            Assert.NotNull(t_supplier);
            t_supplier.Name.AreEqualIgnoringCase(name);
            t_supplier.ShortDescription.AreEqualIgnoringCase(shortDescription);
            t_supplier.Description.AreEqualIgnoringCase(description);
            t_supplier.AreEqual(Processor.Execute(new GetCatalogSupplierTranslation
            {
                EntityID = t_supplier.EntityID
            }).Result);

            name = name.Reverse();
            shortDescription = shortDescription.Reverse();
            description = description.Reverse();
            var upd_t_supplier = Processor.Execute(new UpdateCatalogSupplierTranslation
            {
                EntityID = t_supplier.EntityID,
                LanguageID = t_supplier.Language.EntityID,
                Name = name,
                ShortDescription = shortDescription,
                Description = description
            }).Result;

            Assert.NotNull(upd_t_supplier);
            upd_t_supplier.AreEqual(t_supplier);
            upd_t_supplier.Name.AreEqualIgnoringCase(name);
            upd_t_supplier.ShortDescription.AreEqualIgnoringCase(shortDescription);
            upd_t_supplier.Description.AreEqualIgnoringCase(description);

            Processor.Execute(new DeleteCatalogSupplierTranslation
            {
                EntityID = t_supplier.EntityID
            });
        }
    }
}
