﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class CatalogTagTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name")]
        public void Can_CRUD(string name)
        {
            var t_tag = Processor.Execute(new CreateCatalogTagTranslation
            {
                HolderID = Processor.Execute(new CreateCatalogTag()).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(t_tag);
            t_tag.Name.AreEqualIgnoringCase(name);
            t_tag.AreEqual(Processor.Execute(new GetCatalogTagTranslation
            {
                EntityID = t_tag.EntityID
            }).Result);

            name = name.Reverse();
            var upd_t_tag = Processor.Execute(new UpdateCatalogTagTranslation
            {
                EntityID = t_tag.EntityID,
                LanguageID = t_tag.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_tag);
            upd_t_tag.AreEqual(t_tag);
            upd_t_tag.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteCatalogTagTranslation
            {
                EntityID = t_tag.EntityID
            });
        }
    }
}
