﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class FileGroupTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name")]
        public void Can_CRUD(string name)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var owner = Processor.Execute(new CreateFileOwner
            {
                AccountID = account.EntityID,
                Quota = 1
            }).Result;

            #endregion

            var t_group = Processor.Execute(new CreateFileGroupTranslation
            {
                HolderID = Processor.Execute(new CreateFileGroup
                {
                    OwnerID = owner.EntityID
                }).Result.EntityID,
                LanguageID = localizationProfile.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(t_group);
            t_group.Name.AreEqualIgnoringCase(name);
            t_group.AreEqual(Processor.Execute(new GetFileGroupTranslation
            {
                EntityID = t_group.EntityID
            }).Result);

            name = name.Reverse();
            var upd_t_group = Processor.Execute(new UpdateFileGroupTranslation
            {
                EntityID = t_group.EntityID,
                LanguageID = t_group.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_group);
            upd_t_group.AreEqual(t_group);
            upd_t_group.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteFileGroupTranslation
            {
                EntityID = t_group.EntityID
            });
        }
    }
}
