﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class FixedDiscountRuleTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name", "coupon", 1, 10)]
        public void Can_CRUD(string name, string couponCode, int usageNumber, decimal amount)
        {
            var rule = Processor.Execute(new CreateFixedDiscountRule
            {
                Name = name,
                CommonFlags = SalesRuleFlags.None,
                Flags = FixedDiscountRuleFlags.None,
                CouponCode = couponCode,
                UsageNumber = usageNumber,
                Amount = amount
            }).Result;

            Assert.NotNull(rule);
            rule.Name.AreEqualIgnoringCase(name);
            rule.CouponCode.AreEqualIgnoringCase(couponCode);
            rule.Amount.AreEqual(amount);
            rule.UsageNumber.AreEqual(usageNumber);
            rule.AreEqual(Processor.Execute(new GetSalesRule
            {
                EntityID = rule.EntityID
            }).Result);

            name = name.Reverse();
            couponCode = couponCode.Reverse();
            usageNumber = usageNumber * 2;
            amount = amount * 2;
            var upd_rule = Processor.Execute(new UpdateFixedDiscountRule
            {
                EntityID = rule.EntityID,
                Name = name,
                CommonFlags = SalesRuleFlags.Active,
                Flags = FixedDiscountRuleFlags.Percent,
                Amount = amount,
                CouponCode = couponCode,
                UsageNumber = usageNumber
            }).Result;

            Assert.NotNull(upd_rule);
            upd_rule.AreEqual(rule);
            upd_rule.Name.AreEqualIgnoringCase(name);
            upd_rule.CouponCode.AreEqualIgnoringCase(couponCode);
            upd_rule.Amount.AreEqual(amount);
            upd_rule.UsageNumber.AreEqual(usageNumber);
            upd_rule.CommonFlags.AreEqual(SalesRuleFlags.Active);
            upd_rule.Flags.AreEqual(FixedDiscountRuleFlags.Percent);

            Processor.Execute(new DeleteSalesRule
            {
                EntityID = rule.EntityID
            });
        }

        [TestCase(0)]
        [TestCase(-1)]
        public void Cannot_CreateUpdate_If_Amount_LtOrEq_Zero(decimal amount)
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateFixedDiscountRule
            {
                Amount = amount
            }));
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateFixedDiscountRule
            {
                Amount = amount
            }));
        }
    }
}
