﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class ForumMemberTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("signature")]
        public void Can_CRUD(string signature)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;

            #endregion

            var member = Processor.Execute(new CreateForumMember
            {
                AccountID = account.EntityID,
                AvatarFileID = 0,
                Signature = signature
            }).Result;

            Assert.NotNull(member);
            member.Signature.AreEqualIgnoringCase(signature);
            member.AreEqual(Processor.Execute(new GetForumMember
            {
                EntityID = member.EntityID
            }).Result);

            signature = signature.Reverse();
            var upd_member = Processor.Execute(new UpdateForumMember
            {
                EntityID = member.EntityID,
                Signature = signature,
                Flags = ForumMemberFlags.Active
            }).Result;

            Assert.NotNull(upd_member);
            upd_member.AreEqual(member);
            upd_member.Signature.AreEqualIgnoringCase(signature);
            upd_member.Flags.AreEqual(ForumMemberFlags.Active);

            Processor.Execute(new DeleteForumMember
            {
                EntityID = member.EntityID
            });
        }
    }
}
