﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class ForumRankTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name")]
        public void Can_CRUD(string name)
        {
            var t_rank = Processor.Execute(new CreateForumRankTranslation
            {
                HolderID = Processor.Execute(new CreateForumRank
                {
                    Threshold = 1
                }).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(t_rank);
            t_rank.Name.AreEqualIgnoringCase(name);
            t_rank.AreEqual(Processor.Execute(new GetForumRankTranslation
            {
                EntityID = t_rank.EntityID
            }).Result);

            name = name.Reverse();
            var upd_t_rank = Processor.Execute(new UpdateForumRankTranslation
            {
                EntityID = t_rank.EntityID,
                LanguageID = t_rank.Language.EntityID,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_rank);
            upd_t_rank.AreEqual(t_rank);
            upd_t_rank.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteForumRankTranslation
            {
                EntityID = t_rank.EntityID
            });
        }
    }
}
