﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class LanguageTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("xy", "name")]
        public void Can_CRUD(string code, string name)
        {
            var language = Processor.Execute(new CreateLanguage
            {
                Code = code,
                Name = name,
            }).Result;

            Assert.NotNull(language);
            language.Code.AreEqualIgnoringCase(code);
            language.Name.AreEqualIgnoringCase(name);
            language.AreEqual(Processor.Execute(new GetLanguage
            {
                EntityID = language.EntityID
            }).Result);
            language.AreEqual(Processor.Execute(new GetLanguageByCode
            {
                Code = code
            }).Result);

            code = code.Reverse();
            name = name.Reverse();
            var upd_language = Processor.Execute(new UpdateLanguage
            {
                EntityID = language.EntityID,
                Code = code,
                Name = name,
                Flags = LanguageFlags.Active
            }).Result;

            Assert.NotNull(upd_language);
            upd_language.AreEqual(language);
            upd_language.Code.AreEqualIgnoringCase(code);
            upd_language.Name.AreEqualIgnoringCase(name);
            upd_language.Flags.AreEqual(LanguageFlags.Active);

            Processor.Execute(new DeleteLanguage
            {
                EntityID = language.EntityID
            });
        }

        [Test]
        public void Cannot_Create_If_Code_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateLanguage
            {
                Code = String.Empty
            }));
        }

        [Test]
        public void Cannot_Update_If_Code_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateLanguage
            {
                Code = String.Empty
            }));
        }
    }
}
