﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class LookupAttributeTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name", "value")]
        public void Can_CRUD(string name, string value)
        {
            var t_attribute = Processor.Execute(new CreateLookupAttributeTranslation
            {
                HolderID = Processor.Execute(new CreateLookupAttribute()).Result.EntityID,
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                Value = value,
                Name = name
            }).Result;

            Assert.NotNull(t_attribute);
            t_attribute.Value.AreEqualIgnoringCase(value);
            t_attribute.Name.AreEqualIgnoringCase(name);
            t_attribute.AreEqual(Processor.Execute(new GetLookupAttributeTranslation
            {
                EntityID = t_attribute.EntityID
            }).Result);

            value = value.Reverse();
            name = name.Reverse();
            var upd_t_attribute = Processor.Execute(new UpdateLookupAttributeTranslation
            {
                EntityID = t_attribute.EntityID,
                LanguageID = t_attribute.Language.EntityID,
                Value = value,
                Name = name
            }).Result;

            Assert.NotNull(upd_t_attribute);
            upd_t_attribute.AreEqual(t_attribute);
            upd_t_attribute.Value.AreEqualIgnoringCase(value);
            upd_t_attribute.Name.AreEqualIgnoringCase(name);

            Processor.Execute(new DeleteLookupAttributeTranslation
            {
                EntityID = t_attribute.EntityID
            });
        }
    }
}
