﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class PublicationTranslationTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("title", "description", "text")]
        public void Can_CRUD(string title, string description, string text)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;
            var author = Processor.Execute(new CreatePublicationAuthor
            {
                AccountID = account.EntityID
            }).Result;

            #endregion

            var t_publication = Processor.Execute(new CreatePublicationTranslation
            {
                HolderID = Processor.Execute(new CreatePublication
                {
                    Code = "xy",
                    AuthorID = author.EntityID,
                    PublishedOn = DateTime.UtcNow
                }).Result.EntityID,
                LanguageID = localizationProfile.Language.EntityID,
                Title = title,
                Description = description,
                Text = text
            }).Result;

            Assert.NotNull(t_publication);
            t_publication.Title.AreEqualIgnoringCase(title);
            t_publication.Description.AreEqualIgnoringCase(description);
            t_publication.Text.AreEqualIgnoringCase(text);
            t_publication.AreEqual(Processor.Execute(new GetPublicationTranslation
            {
                EntityID = t_publication.EntityID
            }).Result);

            title = title.Reverse();
            description = description.Reverse();
            text = text.Reverse();
            var upd_t_publication = Processor.Execute(new UpdatePublicationTranslation
            {
                EntityID = t_publication.EntityID,
                LanguageID = t_publication.Language.EntityID,
                Title = title,
                Description = description,
                Text = text
            }).Result;

            Assert.NotNull(upd_t_publication);
            upd_t_publication.AreEqual(t_publication);
            upd_t_publication.Title.AreEqualIgnoringCase(title);
            upd_t_publication.Description.AreEqualIgnoringCase(description);
            upd_t_publication.Text.AreEqualIgnoringCase(text);

            Processor.Execute(new DeletePublicationTranslation
            {
                EntityID = t_publication.EntityID
            });
        }
    }
}
