﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class RoleTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("xy", "name")]
        public void Can_CRUD(string code, string name)
        {
            var role = Processor.Execute(new CreateRole
            {
                Code = code,
                Name = name
            }).Result;

            Assert.NotNull(role);
            role.Code.AreEqualIgnoringCase(code);
            role.Name.AreEqualIgnoringCase(name);
            role.AreEqual(Processor.Execute(new GetRole
            {
                EntityID = role.EntityID
            }).Result);
            role.AreEqual(Processor.Execute(new GetRoleByCode
            {
                Code = code
            }).Result);

            code = code.Reverse();
            name = name.Reverse();
            var upd_role = Processor.Execute(new UpdateRole
            {
                EntityID = role.EntityID,
                Code = code,
                Name = name,
                Flags = RoleFlags.Active
            }).Result;

            Assert.NotNull(upd_role);
            upd_role.AreEqual(role);
            upd_role.Code.AreEqualIgnoringCase(code);
            upd_role.Name.AreEqualIgnoringCase(name);
            upd_role.Flags.AreEqual(RoleFlags.Active);

            Processor.Execute(new DeleteRole
            {
                EntityID = role.EntityID
            });
        }

        [Test]
        public void Cannot_Create_If_Code_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateRole
            {
                Code = String.Empty
            }));
        }

        [Test]
        public void Cannot_Update_If_Code_Empty()
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateRole
            {
                Code = String.Empty
            }));
        }

        [Test]
        public void Can_Manage_Permission_Relations()
        {
            var role = Processor.Execute(new CreateRole
            {
                Code = "xy"
            }).Result;
            var permission = Processor.Execute(new CreatePermission
            {
                Code = "xy"
            }).Result;

            Processor.Execute(new MapPermissionToRole
            {
                EntityID = role.EntityID,
                ChildID = permission.EntityID
            }).Result.Permissions
            .DoesContain(permission);

            Processor.Execute(new UnmapPermissionFromRole
            {
                EntityID = role.EntityID,
                ChildID = permission.EntityID
            }).Result.Permissions
            .DoesNotContain(permission);
        }
    }
}
