﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class SellerTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("displayName")]
        public void Can_CRUD(string displayName)
        {
            #region Prepare

            var country = Processor.Execute(new CreateCountry
            {
                Code = "xy"
            }).Result;
            var localizationProfile = Processor.Execute(new CreateLocalizationProfile
            {
                Code = "xy",
                LanguageID = Processor.Execute(new CreateLanguage
                {
                    Code = "xy"
                }).Result.EntityID,
                TimeZoneID = Processor.Execute(new CreateTimeZone
                {
                    Code = "xy"
                }).Result.EntityID,
                CurrencyID = Processor.Execute(new CreateCurrency
                {
                    Code = "xy",
                    Rate = 1
                }).Result.EntityID,
                CountryID = country.EntityID,
                RegionID = Processor.Execute(new CreateRegion
                {
                    Code = "xy",
                    CountryID = country.EntityID
                }).Result.EntityID,
                DimensionUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-dim",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Dimension
                }).Result.EntityID,
                WeightUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-weight",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Weight
                }).Result.EntityID,
                InformationUnitID = Processor.Execute(new CreateMeasureUnit
                {
                    Code = "xy-info",
                    Ratio = 1,
                    Flags = MeasureUnitFlags.Information
                }).Result.EntityID
            }).Result;
            var account = Processor.Execute(new CreateAccount
            {
                Email = "email@example.com",
                Username = "username",
                Password = "password",
                LocalizationProfileID = localizationProfile.EntityID
            }).Result;

            #endregion

            var seller = Processor.Execute(new CreateSeller
            {
                AccountID = account.EntityID,
                DisplayName = displayName
            }).Result;

            Assert.NotNull(seller);
            seller.DisplayName.AreEqualIgnoringCase(displayName);
            seller.AreEqual(Processor.Execute(new GetSeller
            {
                EntityID = seller.EntityID
            }).Result);
            seller.AreEqual(Processor.Execute(new GetSellerByAccount
            {
                AccountID = account.EntityID
            }).Result);

            displayName = displayName.Reverse();
            var upd_seller = Processor.Execute(new UpdateSeller
            {
                EntityID = seller.EntityID,
                DisplayName = displayName,
                Flags = SellerFlags.Active
            }).Result;

            Assert.NotNull(upd_seller);
            upd_seller.DisplayName.AreEqualIgnoringCase(displayName);
            upd_seller.AreEqual(seller);
            upd_seller.Flags.AreEqual(SellerFlags.Active);

            Processor.Execute(new DeleteSeller
            {
                EntityID = seller.EntityID
            });
        }
    }
}
