﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using Xenta.Enums;
using Xenta.Operations;
using Xenta.Utils;

namespace Xenta.Tests.Operations
{
    [TestFixture]
    public class VolumeDiscountRuleTests : OperationTest
    {
        protected override IEnumerable<Type> ModelMappings
        {
            get
            {
                return NHMappings.ALL;
            }
        }

        [TestCase("name", 1, 2, 10)]
        public void Can_CRUD(string name, decimal thresholdLower, decimal thresholdUpper, decimal amount)
        {
            var rule = Processor.Execute(new CreateVolumeDiscountRule
            {
                Name = name,
                CommonFlags = SalesRuleFlags.None,
                Flags = VolumeDiscountRuleFlags.None,
                ThresholdLower = thresholdLower,
                ThresholdUpper = thresholdUpper,
                Amount = amount
            }).Result;

            Assert.NotNull(rule);
            rule.Name.AreEqualIgnoringCase(name);
            rule.Amount.AreEqual(amount);
            rule.ThresholdLower.AreEqual(thresholdLower);
            rule.ThresholdUpper.AreEqual(thresholdUpper);
            rule.AreEqual(Processor.Execute(new GetSalesRule
            {
                EntityID = rule.EntityID
            }).Result);

            name = name.Reverse();
            thresholdLower = thresholdLower * 2;
            thresholdUpper = thresholdUpper * 2;
            amount = amount * 2;
            var upd_rule = Processor.Execute(new UpdateVolumeDiscountRule
            {
                EntityID = rule.EntityID,
                Name = name,
                CommonFlags = SalesRuleFlags.Active,
                Flags = VolumeDiscountRuleFlags.Percent,
                Amount = amount,
                ThresholdLower = thresholdLower,
                ThresholdUpper = thresholdUpper,
            }).Result;

            Assert.NotNull(upd_rule);
            upd_rule.AreEqual(rule);
            upd_rule.Name.AreEqualIgnoringCase(name);
            rule.ThresholdLower.AreEqual(thresholdLower);
            rule.ThresholdUpper.AreEqual(thresholdUpper);
            upd_rule.Amount.AreEqual(amount);
            upd_rule.CommonFlags.AreEqual(SalesRuleFlags.Active);
            upd_rule.Flags.AreEqual(VolumeDiscountRuleFlags.Percent);

            Processor.Execute(new DeleteSalesRule
            {
                EntityID = rule.EntityID
            });
        }

        [TestCase(0)]
        [TestCase(-1)]
        public void Cannot_CreateUpdate_If_Amount_LtOrEq_Zero(decimal amount)
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateVolumeDiscountRule
            {
                Amount = amount
            }));
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateVolumeDiscountRule
            {
                Amount = amount
            }));
        }

        [TestCase(2, 1)]
        public void Cannot_CreateUpdate_If_ThresholdLower_Lt_ThresholdUpper(decimal thresholdLower, decimal thresholdUpper)
        {
            Assert.Throws<ApplicationException>(() => Processor.Execute(new CreateVolumeDiscountRule
            {
                Amount = 10,
                ThresholdLower = thresholdLower,
                ThresholdUpper = thresholdUpper
            }));
            Assert.Throws<ApplicationException>(() => Processor.Execute(new UpdateVolumeDiscountRule
            {
                Amount = 10,
                ThresholdLower = thresholdLower,
                ThresholdUpper = thresholdUpper
            }));
        }
    }
}
